/*
 * G8RTOS_IPC.c
 *
 *  Created on: Jan 10, 2017
 *      Author: Daniel Gonzalez & David Rivera
 */
#include <G8RTOS_IPC.h>
#include <G8RTOS_Semaphores.h>
#include <stdint.h>
#include "msp.h"

/*********************************************** Defines ******************************************************************************/

#define FIFOSIZE 16
#define MAX_NUMBER_OF_FIFOS 4

/*********************************************** Defines ******************************************************************************/


/*********************************************** Data Structures Used *****************************************************************/

/*
 * FIFO struct will hold
 *  - buffer
 *  - head
 *  - tail
 *  - lost data
 *  - current size
 *  - mutex
 */
typedef struct FIF{
    int32_t Buffer[FIFOSIZE];
    int32_t *Head;
    int32_t *Tail;
    uint32_t LostData;
    semaphore_t CurrentSize;
    semaphore_t Mutex;
}FIFO_t;

/* Array of FIFOS */
static FIFO_t FIFOs[4];

/*********************************************** Data Structures Used *****************************************************************/

/*
 * Initializes FIFO Struct
 */
int G8RTOS_InitFIFO(uint32_t FIFOIndex)
{
    if(FIFOIndex >= MAX_NUMBER_OF_FIFOS){
            return FIFO_LIMIT_REACHED;
    }else{
       FIFOs[FIFOIndex].Head        = &FIFOs[FIFOIndex].Buffer[0];
       FIFOs[FIFOIndex].Tail        = &FIFOs[FIFOIndex].Buffer[0];
       FIFOs[FIFOIndex].LostData    = 0;
       FIFOs[FIFOIndex].CurrentSize = 0;
       FIFOs[FIFOIndex].Mutex       = 1;
    }

    return NO_ERROR;
}

/*
 * Reads FIFO
 *  - Waits until CurrentSize semaphore is greater than zero
 *  - Gets data and increments the head ptr (wraps if necessary)
 * Param: "FIFOChoice": chooses which buffer we want to read from
 * Returns: uint32_t Data from FIFO
 */
uint32_t readFIFO(uint32_t FIFOChoice)
{
    uint32_t fifoData;

    FIFO_t *fifo_ptr = &FIFOs[FIFOChoice];          //fifo_ptr points to FIFOs[Choice]
    G8RTOS_WaitSemaphore(&(fifo_ptr->CurrentSize)); //check if CurrentSize is greater than 0 (data available to read)
    G8RTOS_WaitSemaphore(&(fifo_ptr->Mutex));       //check if current FIFO is in use, if not, claim usage

    fifoData = *(fifo_ptr->Head);                   //Read data pointed to by head

    //incrementing/wrapping *Head
    if(fifo_ptr->Head == &fifo_ptr->Buffer[FIFOSIZE - 1])
    {
        fifo_ptr->Head = &fifo_ptr->Buffer[0];
    }else{
        (fifo_ptr->Head)++;
    }

    G8RTOS_SignalSemaphore(&(fifo_ptr->Mutex));     //Free FIFO
    return fifoData;
}

/*
 *  Writes to FIFO
 *  Writes data to Tail of the buffer if the buffer is not full
 *  Increments tail (wraps if necessary)
 *  Param "FIFOChoice": chooses which buffer we want to read from
 *        "Data': Data being put into FIFO
 *  Returns: error code for full buffer if unable to write
 */
int writeFIFO(uint32_t FIFOChoice, uint32_t Data)
{
    int retVal;

    FIFO_t *fifo_ptr = &FIFOs[FIFOChoice];

    *(fifo_ptr->Tail) = Data;                   //Store data (potentially overwriting) to location of Tail

    if(fifo_ptr->CurrentSize > (FIFOSIZE - 1))  //If no more space is available in FIFO buffer, then overwrite, & increment LostData
    {
        (fifo_ptr->LostData)++;
        retVal = FIFO_DATA_LOST;
    }else{
        G8RTOS_SignalSemaphore(&(fifo_ptr->CurrentSize));   //Increment the CurrentSize
        retVal = NO_ERROR;
    }

    //Incrementing/wrapping *Tail
    if(fifo_ptr->Tail == &fifo_ptr->Buffer[FIFOSIZE - 1])
    {
        fifo_ptr->Tail = &fifo_ptr->Buffer[0];
    }else{
        (fifo_ptr->Tail)++;
    }

    return retVal;

}

