/*
 * G8RTOS_Semaphores.c
 */

/*********************************************** Dependencies and Externs *************************************************************/

#include <G8RTOS_CriticalSection.h>
#include <G8RTOS_Semaphores.h>
#include <stdint.h>
#include "msp.h"

/*********************************************** Dependencies and Externs *************************************************************/


/*********************************************** Public Functions *********************************************************************/

/*
 * Initializes a semaphore to a given value
 * Param "s": Pointer to semaphore
 * Param "value": Value to initialize semaphore to
 * THIS IS A CRITICAL SECTION
 */
void G8RTOS_InitSemaphore(semaphore_t *s, int32_t value)
{
    int32_t IBit_State = StartCriticalSection();    //enter critical section
    *s = value;                                     //initialize the semaphore
    EndCriticalSection(IBit_State);                 //exit critical section, re-enabling interrupts}
}
/*
 * No longer waits for semaphore
 *  - Decrements semaphore
 *  - Blocks thread is sempahore is unavalible
 * Param "s": Pointer to semaphore to wait on
 * THIS IS A CRITICAL SECTION
 */
void G8RTOS_WaitSemaphore(semaphore_t *s)
{
    int32_t IBit_State = StartCriticalSection(); //enter critical section
	(*s)--;                                      //decrement semaphore

	if((*s) < 0)                                 //if resource guarded by semaphore is currently being used
	{
	    CurrentlyRunningThread->Blocked = s;     //Block thread
        EndCriticalSection(IBit_State);          //exit critical section, re-enabling interrupts
        SCB->ICSR |= SCB_ICSR_PENDSVSET_Msk;     //set PendSV pending to perform context switch......don't want to affect SystemTime
	}
    EndCriticalSection(IBit_State);              //exit critical section, re-enabling interrupts}
}

/*
 * Signals the completion of the usage of a semaphore
 *  - Increments the semaphore value by 1
 *  - Unblocks any threads waiting on that semaphore
 * Param "s": Pointer to semaphore to be signaled
 * THIS IS A CRITICAL SECTION
 */
void G8RTOS_SignalSemaphore(semaphore_t *s)
{
    int32_t IBit_State = StartCriticalSection();                //enter critical section
	(*s)++;                                                     //increment semaphore

	if((*s) <= 0)                                               //Check if resource is being used
	{
	    tcb_t *pt = CurrentlyRunningThread->NextTCB;  //pt points to the next thread
	    while(pt->Blocked != s)// && pt != CurrentlyRunningThread)                                 //Find thread that is blocked on the desired resource
	    {
	        pt = pt->NextTCB;
	    }
	    pt->Blocked = 0;                                        //Unblock 1st thread that is blocked on the same semaphore
	}
    EndCriticalSection(IBit_State);                             //exit critical section, re-enabling interrupts}
}

/*********************************************** Public Functions *********************************************************************/


